// This program is in the public domain

#include "record.h"
#include <cstdlib>

const char Record::RS = '\n';

// Set feild widths to the NUMBER integers passed
void Record::setFieldWidths(int number, ...)
{
   va_list ap;
   va_start(ap, number);

   // Reset the object
   septype = FIXED;
   widths.clear();
   fs = "";

   // Set the widths
   while (number-- > 0)
      widths.push_back(va_arg(ap, int));

   va_end(ap);
}

// Standard initializations
void Record::init()
{
   parsed = false;
   records = 0;
   cfield = 0;
   ownfile = false;
}

bool Record::read()
{
   char c;

   // Reset the record
   str = "";
   cfield = 1;
   parsed = false;

   if (file->eof())
      return false;

   while (1) {
      c = file->get();
      if (c == RS || file->eof()) break;
      str += c;
   }

   records++;
   return true;
}

// Parse the current record into fields
void Record::parse()
{
   unsigned pos = 0;             // str position
   unsigned len = str.length();  // str length

   fields.clear();   // Reset the fields

   if (septype == FIXED) {
      unsigned i = 0;
      while (i < widths.size() && pos < len) {
         fields.push_back(str.substr(pos, widths[i]));
         pos += widths[i];
         i++;
      }
   }
   else if (septype == STRING) {
      unsigned long index;
      do {
         index = str.find(fs, pos);
         if (index != string::npos) {
            fields.push_back(str.substr(pos, index-pos));
            pos = index + fs.length();
         }
      } while (index != string::npos);
   }

   if (pos < str.length())  // Get any remaining characters at rhe end
      fields.push_back(str.substr(pos));

   parsed = true;
   return;
}

string Record::field(int n) {
   if (!parsed) parse();

   if (n == -1)   // (-1) -- last field
      return fields[fields.size()-1];
   if (n > (signed)fields.size() || n < 0)  // Out of bounds -- null string
      return string();
   if (n == 0)    // 0 -- entire record
      return str;

   return fields[n - 1];
}

int Record::ifield(int n)
{
   string s = field(n);
   return atoi(s.c_str());
}

double Record::ffield(int n)
{
   string s = field(n);
   return atof(s.c_str());
}

/*
// Assign to the fields
void Record::setfield(int n, string value);
void Record::setfield(int n, int value);
void Record::setfield(int n, double value, int precision=6);

// Insert after
void Record::insert(int n, string value);
void Record::insert(int n, int value);
void Record::insert(int n, double value, int precision=6);

void Record::delete(int n);
*/
