// Record -- read records from a stream and parse into fields
// TODO:
//    Regular expressions
//    Record separator/record size
//    modify/add/delete fields ?
// This program is in the public domain

#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include <cstdarg>

using namespace std;

typedef vector<int> ivector;
typedef vector<string> svector;

class Record {
public:
   enum FSType {FIXED, STRING};

   // Constructors
   Record(istream *file, const ivector& widths) {
      this->file = file;
      this->septype = FIXED;
      this->widths = widths;
      init();
   }
   Record(istream *file, const string& sep) {
      this->file = file;
      this->septype = STRING;
      this->fs = sep;
      init();
   }
   Record(const string& filename, const ivector& widths) {
      file = new ifstream(filename.c_str());
      Record(file, widths);
      ownfile = true;
   }
   Record(const string& filename, const string& sep) {
      file = new ifstream(filename.c_str());
      Record(file, sep);
      ownfile = true;
   }

   //Destructor
   ~Record() { if (ownfile) delete file; }

   // Set the field separator
   void setFS(const string& sep) {
      widths.clear();
      septype = STRING;
      fs = sep;
      parsed = false;
   }

   // Set the field widths
   void setFieldWidths(const ivector& widths) {
      fs = "";
      septype = FIXED;
      this->widths = widths;
      parsed = false;
   }
   // Variable argument version
   void setFieldWidths(int number, ...);

   // Properties
   FSType type()  const { return septype; }     // How the fields are separated
   int nfields() { if (!parsed) parse(); return fields.size(); }
                                                // Number of fields
   int nrecords() const { return records; }     // Number of records read
   bool eof()     const { return file->eof(); } // Is the file at EOF?

   // Methods
   bool read();               // Read the next record
   string field(int n);       // Get a string field
   string operator[](int n) { return field(n); }
   int ifield(int n);         // Get an integer field
   double ffield(int n);      // Get a floating point field
   svector allfields() { return fields; }
                              // Return all of the fields in a vector

   void rewind() { cfield = 1; }             // Set the current field back to 1
   string next() { return field(cfield++); } // Get the next field and advance
   string last() { return field(-1); }       // Get the last field
   bool end() const { return cfield >= fields.size(); }
                                             // Is the current field the last one?

   // Assign to the fields
   //void setfield(int n, string value);
   //void setfield(int n, int value);
   //void setfield(int n, double value, int precision=6);

   // Insert after
   //void insert(int n, string value);
   //void insert(int n, int value);
   //void insert(int n, double value, int precision=6);

   // Delete
   //void remove(int n);


private:
   istream *file;    // The input stream
   bool ownfile;     // Did we open the file?
   string str;       // The string containing the whole record

   FSType septype;   // FIXED or STRING separated fields
   ivector widths;   // FIXED field widths
   string fs;        // Field separator

   bool parsed;      // Is the current record parsed into fields?
   svector fields;   // The fields
   int records;      // Number of records read so far
   unsigned cfield;  // Current field

   void init();      // Initialize object fields
   void parse();     // Parse the record into fields

   static const char RS;
};
